"""
Module for safely evaluating arithmetic expressions in strings.

http://stackoverflow.com/a/9558001
"""

import ast
import operator as op

# supported operators
operators = {ast.Add: op.add, ast.Sub: op.sub, ast.Mult: op.mul,
             ast.Div: op.truediv, ast.Pow: op.pow, ast.BitXor: op.xor,
             ast.USub: op.neg}


def eval_expr(expr, variables=None):
    """
    >>> eval_expr('2^6')
    4
    >>> eval_expr('2**6')
    64
    >>> eval_expr('1 + 2*3**(4^5) / (6 + -7)')
    -5.0
    """
    if variables is None:
        variables = {}
    return eval_(ast.parse(expr, mode='eval').body, variables)


def eval_(node, variables):
    if isinstance(node, ast.Num):  # <number>
        return node.n
    elif isinstance(node, ast.BinOp):  # <left> <operator> <right>
        return operators[type(node.op)](eval_(node.left, variables),
                                        eval_(node.right, variables))
    elif isinstance(node, ast.UnaryOp):  # <operator> <operand> e.g., -1
        return operators[type(node.op)](eval_(node.operand, variables))
    elif isinstance(node, ast.Name):
        return variables[node.id]
    else:
        raise TypeError(node)
